#!/usr/bin/perl -w

########################################################
# 
# cfgCreator.pl
#    part of the ncDNAlign alignment package
#
# Authors: Dominic Rose
#
# Contact: dominic@bioinf.uni-leipzig.de
#
########################################################

use strict;
use Getopt::Long;
use Pod::Usage;

my ($opt_out, $opt_org, $opt_help, $opt_man);

GetOptions(
	"c|cfg=s" 	=> \$opt_out,
	"o|org=i"	=> \$opt_org,
	"h|help!"	=> \$opt_help,
	"man!"		=> \$opt_man
) or pod2usage(2);

pod2usage(1) if ($opt_help);
pod2usage(-exitstatus => 0, -verbose => 2) if $opt_man;
pod2usage(1) if(!$opt_out || !$opt_org);

# ------ READ IN ----------
my @in;
my @jobs;
my %org;


print "\n----- >>>>>   ncDNAlign - cfgCreator   <<<<< ------\n";
print  "   creating custom ncDNAlign configuration files\n";
print  "appropriate default values are given in parentheses\n";
print  "---------------------------------------------------\n\n";


$jobs[0] = "Specify the projects' working directory: ";
$jobs[1] = "Specify a directory containing your source files: ";
$jobs[2] = "Specify an output directory which will contain produced alignments: ";
$jobs[3] = "Specify the blast call (e.g.: blastall -p blastn): ";
$jobs[4] = "Specify the blast eValue (e.g.: 1e-3): ";
$jobs[5] = "Specify the alignment call (e.g.: dialign2-2 -n): ";
$jobs[6] = "Specify the minimal number of sequences per alignment (e.g.: 3): ";
$jobs[7] = "Specify the minimal alignment length (e.g.: 40): ";
$jobs[8] = "Specify GenBank feature keys that should be rejected\n".
	"(e.g.: CDS;repeat_region;repeat_unit;LTR;satellite;sig_peptide;transit_peptide;mat_peptide): ";
$jobs[9] = "Specify the maximal relative frequency of gaps per sequence (e.g. 0.3): ";
$jobs[10] = "Specify the name of the reference organism (3 letter code): ";
$jobs[11] = "Specify the length of flanking regions that will be appended to your alignments (e.g.: 30): ";


for(my $i=0; $i<scalar(@jobs); $i++) {
	print $jobs[$i];
	$in[$i] = <STDIN>;
	chomp $in[$i];
	
	while(length($in[$i]) == 0) {
		print	"ERRROR: You did entry a correct value, please re-entry: \n";
		print $jobs[$i];
		$in[$i] = <STDIN>;
		chomp $in[$i];
	}
}


# Consistency checks
my $which = 0;
while( $in[$which] !~ /^\/$/) {
	print	"ERRROR: You did not specify the 'projects working directory' correctly, please re-entry (absolute path, starting with \/): ";
	$in[$which] = <STDIN>;
	chomp $in[$which];
}
$which = 1;
while( $in[$which] !~ /^\/$/) {
	print	"ERRROR: You did not specify the 'directory containing your source files' correctly, please re-entry (absolute path, starting with \/): ";
	$in[$which] = <STDIN>;
	chomp $in[$which];
}
$which = 2;
while( $in[$which] !~ /^\/$/) {
	print	"ERRROR: You did not specify the 'output directory which will contain produced alignments' correctly, please re-entry (absolute path, starting with \/): ";
	$in[$which] = <STDIN>;
	chomp $in[$which];
}
$which = 6;
while( $in[$which] !~ /^\d+$/) {
	print	"ERRROR: You did not specify the 'minimal number of sequences per alignment' correctly, please re-entry: ";
	$in[$which] = <STDIN>;
	chomp $in[$which];
}
$which = 7;
while( $in[$which] !~ /^\d+$/) {
	print	"ERRROR: You did not specify the 'minimal alignment length' correctly, please re-entry: ";
	$in[$which] = <STDIN>;
	chomp $in[$which];
}
$which = 10;
while(length($in[$which]) != 3) {
	print "ERRROR: You did not specify the 'name of the reference organism' correctly, please re-entry  (3 letter code): ";
	$in[$which] = <STDIN>;
	chomp $in[$which];
}
$which = 11;
while( $in[$which] !~ /^\d+$/) {
	print	"ERRROR: You did not specify the 'length of flanking regions' correctly, please re-entry: ";
	$in[$which] = <STDIN>;
	chomp $in[$which];
}



# The organism data
for(my $i=1; $i<=$opt_org; $i++) {
	print "--- $i. Organism ---\n";
	print "Name: ";
	$org{$i}{name} = <STDIN>;
	chomp $org{$i}{name};
	
	# length check
	while(length($org{$i}{name}) == 0) {
		print "ERRROR: You did entry a correct value, please re-entry: \n";
		print "Name: ";
		$org{$i}{name} = <STDIN>;
		chomp $org{$i}{name};
	}
	
	print "Output file that will contain nc regions (e.g. xyz.cut.fa): ";
	$org{$i}{file} = <STDIN>;
	chomp $org{$i}{file};
	
	# length check
	while(length($org{$i}{file}) == 0) {
		print "ERRROR: You did entry a correct value, please re-entry: \n";
		print "Output fasta file that will contain nc regions: ";
		$org{$i}{file} = <STDIN>;
		chomp $org{$i}{file};
	}
}


# -------- OUTPUT --------

open (OUT, "> $opt_out") or die "ERROR: Can not open outfile='$opt_out'!";

print OUT "################################\n";
print OUT "# NcDNAlign configuration file #\n";
print OUT "################################\n\n";

print OUT "# ------- Global options -------\n";
print OUT "C_P_DIR             = \"$in[0]\"\n";
print OUT "C_SOURCE_DIR        = \"$in[1]\"\n";
print OUT "C_ALN_DIR           = \"$in[2]\"\n";
print OUT "C_BLAST_CMD         = \"$in[3]\"\n";
print OUT "C_BLAST_EVALUE      = \"$in[4]\"\n";
print OUT "C_ALN_CMD           = \"$in[5]\"\n";
print OUT "C_MIN_SEQ           = \"$in[6]\"\n";
print OUT "C_MIN_ALN_LENGTH    = \"$in[7]\"\n";
print OUT "C_GBK_KEYS          = \"$in[8]\"\n";
print OUT "C_GAP_SCORE         = \"$in[9]\"\n";
print OUT "C_TARGET_NAME       = \"$in[10]\"\n\n";
	
foreach my $k (sort {$a<=>$b} keys %org) {
	print OUT "# ------- $k. Organism   -------\n";	
	print OUT "ORG_NAME            = \"$org{$k}{name}\"\n";
	print OUT "ORG_NCREGIONS_FASTA = \"$org{$k}{file}\"\n";	
}

close OUT;

print "Ready! CFG file available at ==> '$opt_out'\n";

__END__

=head1 NAME

C<cfgCreator.pl> - Creates configuration files for the NcDNAlign package in a user friendly wrapped dialogue

=head1 SYNOPSIS

cfgCreator.pl [options]

=head1 OPTIONS

=over 8

=item B<-c, --cfg> FILE

The configuration data is stored in FILE. [REQUIRED]

=item B<-o, --org> FILE

The number of organisms participating the screen. [REQUIRED]

=back

=head1 DESCRIPTION

Just follow the dialogue to create you config file. The configuration file is written to the specified file (cp. option --cfg). 

=head1 EXAMPLES

# cfgCreator.pl -c myScreen.cfg -o 6
 
=head1 AUTHORS

Dominic Rose (dominic@bioinf.uni-leipzig.de)

=head1 AVAILABILITY

http://www.bioinf.uni-leipzig.de/Software/NcDNAlign/

=cut
